@@ -2,6 +2,7 @@ package gen
 
 import (
 	"fmt"
+	"slices"
 	"strings"
 )
 
@@ -99,6 +100,18 @@ const (
 	AUint32
 	ABool
 
+	// Binary marshaler types
+	BinaryMarshaler // encoding.BinaryMarshaler/BinaryUnmarshaler
+	BinaryAppender  // encoding.BinaryAppender/BinaryUnmarshaler
+
+	// Text marshaler types (stored as binary by default)
+	TextMarshalerBin // encoding.TextMarshaler/TextUnmarshaler -> bin
+	TextAppenderBin  // encoding.TextAppender/TextUnmarshaler -> bin
+
+	// Text marshaler types (stored as string)
+	TextMarshalerString // encoding.TextMarshaler/TextUnmarshaler -> string
+	TextAppenderString  // encoding.TextAppender/TextUnmarshaler -> string
+
 	IDENT // IDENT means an unrecognized identifier
 )
 
@@ -159,14 +172,42 @@ type GenericTypeParams struct {
 	isPtr        bool
 }
 
-func (c *common) SetVarname(s string)                { c.vname = s }
-func (c *common) Varname() string                    { return c.vname }
+func (c *common) SetVarname(s string) { c.vname = s }
+func (c *common) Varname() string     { return c.vname }
+
+// typeNameWithParams returns the type name with generic parameters appended if they exist
+// stripTypeParams removes type parameters from a type name for lookup purposes
+// e.g. "MyType[T, U]" becomes "MyType", "*SomeType[A]" becomes "*SomeType"
+func stripTypeParams(typeName string) string {
+	if idx := strings.Index(typeName, "["); idx != -1 {
+		return typeName[:idx]
+	}
+	return typeName
+}
+
+func (c *common) typeNameWithParams(baseName string) string {
+	if c.typeParams.TypeParams != "" && !strings.Contains(baseName, "[") {
+		// Check if baseName is a single identifier without dots (likely a type parameter)
+		if !strings.Contains(baseName, ".") && len(baseName) <= 2 && len(baseName) > 0 {
+			// This looks like a simple type parameter, don't add type parameters
+			return baseName
+		}
+		return baseName + c.typeParams.TypeParams
+	}
+	return baseName
+}
+
+// baseTypeName returns the type name without generic parameters (for use in method receivers)
+func (c *common) baseTypeName() string {
+	return c.alias
+}
 func (c *common) Alias(typ string)                   { c.alias = typ }
 func (c *common) hidden()                            {}
 func (c *common) AllowNil() bool                     { return false }
 func (c *common) SetIsAllowNil(bool)                 {}
 func (c *common) SetTypeParams(tp GenericTypeParams) { c.typeParams = tp }
 func (c *common) TypeParams() GenericTypeParams      { return c.typeParams }
+func (c *common) BaseTypeName() string               { return c.baseTypeName() }
 func (c *common) AlwaysPtr(set *bool) bool {
 	if c != nil && set != nil {
 		c.ptrRcv = *set
@@ -244,6 +285,9 @@ type Elem interface {
 	// TypeParams returns the generic type parameters for this element
 	TypeParams() GenericTypeParams
 
+	// BaseTypeName returns the type name without generic parameters
+	BaseTypeName() string
+
 	hidden()
 }
 
@@ -282,10 +326,10 @@ ridx:
 
 func (a *Array) TypeName() string {
 	if a.alias != "" {
-		return a.alias
+		return a.typeNameWithParams(a.alias)
 	}
 	a.Alias(fmt.Sprintf("[%s]%s", a.Size, a.Els.TypeName()))
-	return a.alias
+	return a.typeNameWithParams(a.alias)
 }
 
 func (a *Array) Copy() Elem {
@@ -334,14 +378,14 @@ ridx:
 
 func (m *Map) TypeName() string {
 	if m.alias != "" {
-		return m.alias
+		return m.typeNameWithParams(m.alias)
 	}
 	keyType := "string"
 	if m.Key != nil {
 		keyType = m.Key.TypeName()
 	}
 	m.Alias("map[" + keyType + "]" + m.Value.TypeName())
-	return m.alias
+	return m.typeNameWithParams(m.alias)
 }
 
 func (m *Map) Copy() Elem {
@@ -402,10 +446,10 @@ func (s *Slice) SetVarname(a string) {
 
 func (s *Slice) TypeName() string {
 	if s.alias != "" {
-		return s.alias
+		return s.typeNameWithParams(s.alias)
 	}
 	s.Alias("[]" + s.Els.TypeName())
-	return s.alias
+	return s.typeNameWithParams(s.alias)
 }
 
 func (s *Slice) Copy() Elem {
@@ -459,7 +503,10 @@ func (s *Ptr) SetVarname(a string) {
 
 	case *BaseElem:
 		// identities have pointer receivers
-		if x.Value == IDENT {
+		// marshaler types also have pointer receivers
+		if x.Value == IDENT || x.Value == BinaryMarshaler || x.Value == BinaryAppender ||
+			x.Value == TextMarshalerBin || x.Value == TextAppenderBin ||
+			x.Value == TextMarshalerString || x.Value == TextAppenderString {
 			// replace directive sets Convert=true and Needsref=true
 			// since BaseElem is behind a pointer we set Needsref=false
 			if x.Convert {
@@ -479,10 +526,10 @@ func (s *Ptr) SetVarname(a string) {
 
 func (s *Ptr) TypeName() string {
 	if s.alias != "" {
-		return s.alias
+		return s.typeNameWithParams(s.alias)
 	}
 	s.Alias("*" + s.Value.TypeName())
-	return s.alias
+	return s.typeNameWithParams(s.alias)
 }
 
 func (s *Ptr) Copy() Elem {
@@ -594,6 +641,7 @@ type StructField struct {
 	RawTag        string   // the full struct tag
 	FieldName     string   // the name of the struct field
 	FieldElem     Elem     // the field type
+	FieldLimit    uint32   // field-specific size limit for slices/maps (0 = no limit)
 }
 
 // HasTagPart returns true if the specified tag part (option) is present.
@@ -601,12 +649,22 @@ func (sf *StructField) HasTagPart(pname string) bool {
 	if len(sf.FieldTagParts) < 2 {
 		return false
 	}
-	for _, p := range sf.FieldTagParts[1:] {
-		if p == pname {
-			return true
+	return slices.Contains(sf.FieldTagParts[1:], pname)
+}
+
+// GetTagValue returns the value for a tag part with the format "key=value".
+// Returns the value string and true if found, empty string and false if not found.
+func (sf *StructField) GetTagValue(key string) (string, bool) {
+	if len(sf.FieldTagParts) < 2 {
+		return "", false
+	}
+	prefix := key + "="
+	for _, part := range sf.FieldTagParts[1:] {
+		if strings.HasPrefix(part, prefix) {
+			return strings.TrimPrefix(part, prefix), true
 		}
 	}
-	return false
+	return "", false
 }
 
 type ShimMode int
@@ -630,6 +688,7 @@ type BaseElem struct {
 	zerocopy     bool      // Allow zerocopy for byte slices in unmarshal.
 	mustinline   bool      // must inline; not printable
 	needsref     bool      // needs reference for shim
+	parentIsPtr  bool      // parent is a pointer
 	allowNil     *bool     // Override from parent.
 }
 
@@ -677,10 +736,10 @@ func (s *BaseElem) SetVarname(a string) {
 // type name for the base element.
 func (s *BaseElem) TypeName() string {
 	if s.alias != "" {
-		return s.alias
+		return s.typeNameWithParams(s.alias)
 	}
 	s.common.Alias(s.BaseType())
-	return s.alias
+	return s.typeNameWithParams(s.alias)
 }
 
 // ToBase, used if Convert==true, is used as tmp = {{ToBase}}({{Varname}})
@@ -719,7 +778,7 @@ func (s *BaseElem) BaseName() string {
 func (s *BaseElem) BaseType() string {
 	switch s.Value {
 	case IDENT:
-		return s.TypeName()
+		return s.alias
 
 	// exceptions to the naming/capitalization
 	// rule:
@@ -895,6 +954,18 @@ func (k Primitive) String() string {
 		return "atomic.Uint32"
 	case ABool:
 		return "atomic.Bool"
+	case BinaryMarshaler:
+		return "BinaryMarshaler"
+	case BinaryAppender:
+		return "BinaryAppender"
+	case TextMarshalerBin:
+		return "TextMarshalerBin"
+	case TextAppenderBin:
+		return "TextAppenderBin"
+	case TextMarshalerString:
+		return "TextMarshalerString"
+	case TextAppenderString:
+		return "TextAppenderString"
 	case IDENT:
 		return "Ident"
 	default:
