@@ -247,7 +247,14 @@ func (s *sizeGen) gBase(b *BaseElem) {
 		if b.typeParams.isPtr {
 			dst = "*" + dst
 		}
-		if remap := b.typeParams.ToPointerMap[dst]; remap != "" {
+
+		// Strip type parameters from dst for lookup in ToPointerMap
+		lookupKey := stripTypeParams(dst)
+		if idx := strings.Index(dst, "["); idx != -1 {
+			lookupKey = dst[:idx]
+		}
+
+		if remap := b.typeParams.ToPointerMap[lookupKey]; remap != "" {
 			vname = fmt.Sprintf(remap, vname)
 		}
 		s.addConstant(basesizeExpr(b.Value, vname, b.BaseName()))
