@@ -1,8 +1,10 @@
 package msgp
 
 import (
+	"encoding"
 	"encoding/binary"
 	"encoding/json"
+	"fmt"
 	"io"
 	"math"
 	"strconv"
@@ -13,7 +15,7 @@ import (
 )
 
 // where we keep old *Readers
-var readerPool = sync.Pool{New: func() interface{} { return &Reader{} }}
+var readerPool = sync.Pool{New: func() any { return &Reader{} }}
 
 // Type is a MessagePack wire type,
 // including this package's built-in
@@ -152,6 +154,10 @@ type Reader struct {
 	R              *fwd.Reader
 	scratch        []byte
 	recursionDepth int
+
+	maxRecursionDepth int    // maximum recursion depth
+	maxElements       uint32 // maximum number of elements in arrays and maps
+	maxStrLen         uint64 // maximum number of bytes in any string
 }
 
 // Read implements `io.Reader`
@@ -171,7 +177,7 @@ func (m *Reader) CopyNext(w io.Writer) (int64, error) {
 	// Opportunistic optimization: if we can fit the whole thing in the m.R
 	// buffer, then just get a pointer to that, and pass it to w.Write,
 	// avoiding an allocation.
-	if int(sz) <= m.R.BufferSize() {
+	if int(sz) >= 0 && int(sz) <= m.R.BufferSize() {
 		var nn int
 		var buf []byte
 		buf, err = m.R.Next(int(sz))
@@ -203,7 +209,7 @@ func (m *Reader) CopyNext(w io.Writer) (int64, error) {
 		defer done()
 	}
 	// for maps and slices, read elements
-	for x := uintptr(0); x < o; x++ {
+	for range o {
 		var n2 int64
 		n2, err = m.CopyNext(w)
 		if err != nil {
@@ -214,10 +220,53 @@ func (m *Reader) CopyNext(w io.Writer) (int64, error) {
 	return n, nil
 }
 
+// SetMaxRecursionDepth sets the maximum recursion depth.
+func (m *Reader) SetMaxRecursionDepth(d int) {
+	m.maxRecursionDepth = d
+}
+
+// GetMaxRecursionDepth returns the maximum recursion depth.
+// Set to 0 to use the default value of 100000.
+func (m *Reader) GetMaxRecursionDepth() int {
+	if m.maxRecursionDepth <= 0 {
+		return recursionLimit
+	}
+	return m.maxRecursionDepth
+}
+
+// SetMaxElements sets the maximum number of elements to allow in map, bin, array or extension payload.
+// Setting this to 0 will allow any number of elements - math.MaxUint32.
+// This does currently apply to generated code.
+func (m *Reader) SetMaxElements(d uint32) {
+	m.maxElements = d
+}
+
+// GetMaxElements will return the maximum number of elements in a map, bin, array or extension payload.
+func (m *Reader) GetMaxElements() uint32 {
+	if m.maxElements <= 0 {
+		return math.MaxUint32
+	}
+	return m.maxElements
+}
+
+// SetMaxStringLength sets the maximum number of bytes to allow in strings.
+// Setting this == 0 will allow any number of elements - math.MaxUint64.
+func (m *Reader) SetMaxStringLength(d uint64) {
+	m.maxStrLen = d
+}
+
+// GetMaxStringLength will return the current string length limit.
+func (m *Reader) GetMaxStringLength() uint64 {
+	if m.maxStrLen <= 0 {
+		return math.MaxUint64
+	}
+	return min(m.maxStrLen, math.MaxUint64)
+}
+
 // recursiveCall will increment the recursion depth and return an error if it is exceeded.
 // If a nil error is returned, done must be called to decrement the counter.
 func (m *Reader) recursiveCall() (done func(), err error) {
-	if m.recursionDepth >= recursionLimit {
+	if m.recursionDepth >= m.GetMaxRecursionDepth() {
 		return func() {}, ErrRecursion
 	}
 	m.recursionDepth++
@@ -415,7 +464,11 @@ func (m *Reader) ReadMapKey(scratch []byte) ([]byte, error) {
 	out, err := m.ReadStringAsBytes(scratch)
 	if err != nil {
 		if tperr, ok := err.(TypeError); ok && tperr.Encoded == BinType {
-			return m.ReadBytes(scratch)
+			key, err := m.ReadBytes(scratch)
+			if uint64(len(key)) > m.GetMaxStringLength() {
+				return nil, ErrLimitExceeded
+			}
+			return key, err
 		}
 		return nil, err
 	}
@@ -468,6 +521,9 @@ fill:
 	if read == 0 {
 		return nil, ErrShortBytes
 	}
+	if uint64(read) > m.GetMaxStringLength() {
+		return nil, ErrLimitExceeded
+	}
 	return m.R.Next(read)
 }
 
@@ -528,7 +584,7 @@ func (m *Reader) ReadFloat64() (f float64, err error) {
 	var p []byte
 	p, err = m.R.Peek(9)
 	if err != nil {
-		// we'll allow a coversion from float32 to float64,
+		// we'll allow a conversion from float32 to float64,
 		// since we don't lose any precision
 		if err == io.EOF && len(p) > 0 && p[0] == mfloat32 {
 			ef, err := m.ReadFloat32()
@@ -816,7 +872,7 @@ func (m *Reader) ReadUint64() (u uint64, err error) {
 		if err != nil {
 			return
 		}
-		v := int64(getMint64(p))
+		v := getMint64(p)
 		if v < 0 {
 			err = UintBelowZero{Value: v}
 			return
@@ -941,6 +997,10 @@ func (m *Reader) ReadBytes(scratch []byte) (b []byte, err error) {
 		return
 	}
 	if int64(cap(scratch)) < read {
+		if read > int64(m.GetMaxElements()) {
+			err = ErrLimitExceeded
+			return
+		}
 		b = make([]byte, read)
 	} else {
 		b = scratch[0:read]
@@ -980,10 +1040,10 @@ func (m *Reader) ReadBytesHeader() (sz uint32, err error) {
 		if err != nil {
 			return
 		}
-		sz = uint32(big.Uint32(p[1:]))
+		sz = big.Uint32(p[1:])
 		return
 	default:
-		err = badPrefix(BinType, p[0])
+		err = badPrefix(BinType, lead)
 		return
 	}
 }
@@ -1052,7 +1112,7 @@ func (m *Reader) ReadStringAsBytes(scratch []byte) (b []byte, err error) {
 		if err != nil {
 			return
 		}
-		read = int64(uint8(p[1]))
+		read = int64(p[1])
 	case mstr16:
 		p, err = m.R.Next(3)
 		if err != nil {
@@ -1070,6 +1130,10 @@ func (m *Reader) ReadStringAsBytes(scratch []byte) (b []byte, err error) {
 		return
 	}
 fill:
+	if uint64(read) > m.GetMaxStringLength() {
+		err = ErrLimitExceeded
+		return
+	}
 	if int64(cap(scratch)) < read {
 		b = make([]byte, read)
 	} else {
@@ -1143,7 +1207,7 @@ func (m *Reader) ReadString() (s string, err error) {
 		if err != nil {
 			return
 		}
-		read = int64(uint8(p[1]))
+		read = int64(p[1])
 	case mstr16:
 		p, err = m.R.Next(3)
 		if err != nil {
@@ -1165,6 +1229,11 @@ fill:
 		s, err = "", nil
 		return
 	}
+	if uint64(read) > m.GetMaxStringLength() {
+		err = ErrLimitExceeded
+		return
+	}
+
 	// reading into the memory
 	// that will become the string
 	// itself has vastly superior
@@ -1235,7 +1304,7 @@ func (m *Reader) ReadComplex128() (f complex128, err error) {
 
 // ReadMapStrIntf reads a MessagePack map into a map[string]interface{}.
 // (You must pass a non-nil map into the function.)
-func (m *Reader) ReadMapStrIntf(mp map[string]interface{}) (err error) {
+func (m *Reader) ReadMapStrIntf(mp map[string]any) (err error) {
 	var sz uint32
 	sz, err = m.ReadMapHeader()
 	if err != nil {
@@ -1244,9 +1313,13 @@ func (m *Reader) ReadMapStrIntf(mp map[string]interface{}) (err error) {
 	for key := range mp {
 		delete(mp, key)
 	}
+	if sz > m.GetMaxElements() {
+		err = ErrLimitExceeded
+		return
+	}
 	for i := uint32(0); i < sz; i++ {
 		var key string
-		var val interface{}
+		var val any
 		key, err = m.ReadString()
 		if err != nil {
 			return
@@ -1260,6 +1333,13 @@ func (m *Reader) ReadMapStrIntf(mp map[string]interface{}) (err error) {
 	return
 }
 
+// ReadTimeUTC reads a time.Time object from the reader.
+// The returned time's location will be set to UTC.
+func (m *Reader) ReadTimeUTC() (t time.Time, err error) {
+	t, err = m.ReadTime()
+	return t.UTC(), err
+}
+
 // ReadTime reads a time.Time object from the reader.
 // The returned time's location will be set to time.Local.
 func (m *Reader) ReadTime() (t time.Time, err error) {
@@ -1369,7 +1449,7 @@ func (m *Reader) ReadJSONNumber() (n json.Number, err error) {
 // Arrays are decoded as []interface{}, and maps are decoded
 // as map[string]interface{}. Integers are decoded as int64
 // and unsigned integers are decoded as uint64.
-func (m *Reader) ReadIntf() (i interface{}, err error) {
+func (m *Reader) ReadIntf() (i any, err error) {
 	var t Type
 	t, err = m.NextType()
 	if err != nil {
@@ -1439,7 +1519,7 @@ func (m *Reader) ReadIntf() (i interface{}, err error) {
 			defer done()
 		}
 
-		mp := make(map[string]interface{})
+		mp := make(map[string]any)
 		err = m.ReadMapStrIntf(mp)
 		i = mp
 		return
@@ -1470,8 +1550,12 @@ func (m *Reader) ReadIntf() (i interface{}, err error) {
 		} else {
 			defer done()
 		}
+		if sz > m.GetMaxElements() {
+			err = ErrLimitExceeded
+			return
+		}
 
-		out := make([]interface{}, int(sz))
+		out := make([]any, int(sz))
 		for j := range out {
 			out[j], err = m.ReadIntf()
 			if err != nil {
@@ -1485,3 +1569,51 @@ func (m *Reader) ReadIntf() (i interface{}, err error) {
 		return nil, fatal // unreachable
 	}
 }
+
+// ReadBinaryUnmarshal reads a binary-encoded object from the reader and unmarshals it into dst.
+func (m *Reader) ReadBinaryUnmarshal(dst encoding.BinaryUnmarshaler) (err error) {
+	defer func() {
+		if r := recover(); r != nil {
+			err = fmt.Errorf("msgp: panic during UnmarshalBinary: %v", r)
+		}
+	}()
+	tmp := bytesPool.Get().([]byte)
+	defer bytesPool.Put(tmp) //nolint:staticcheck
+	tmp, err = m.ReadBytes(tmp[:0])
+	if err != nil {
+		return
+	}
+	return dst.UnmarshalBinary(tmp)
+}
+
+// ReadTextUnmarshal reads a text-encoded bin array from the reader and unmarshals it into dst.
+func (m *Reader) ReadTextUnmarshal(dst encoding.TextUnmarshaler) (err error) {
+	defer func() {
+		if r := recover(); r != nil {
+			err = fmt.Errorf("msgp: panic during UnmarshalText: %v", r)
+		}
+	}()
+	tmp := bytesPool.Get().([]byte)
+	defer bytesPool.Put(tmp) //nolint:staticcheck
+	tmp, err = m.ReadBytes(tmp[:0])
+	if err != nil {
+		return
+	}
+	return dst.UnmarshalText(tmp)
+}
+
+// ReadTextUnmarshalString reads a text-encoded string from the reader and unmarshals it into dst.
+func (m *Reader) ReadTextUnmarshalString(dst encoding.TextUnmarshaler) (err error) {
+	defer func() {
+		if r := recover(); r != nil {
+			err = fmt.Errorf("msgp: panic during UnmarshalText: %v", r)
+		}
+	}()
+	tmp := bytesPool.Get().([]byte)
+	defer bytesPool.Put(tmp) //nolint:staticcheck
+	tmp, err = m.ReadStringAsBytes(tmp[:0])
+	if err != nil {
+		return
+	}
+	return dst.UnmarshalText(tmp)
+}
