@@ -1138,11 +1138,11 @@ func ReadTimeBytes(b []byte) (t time.Time, o []byte, err error) {
 // ReadMapStrIntfBytes reads a map[string]interface{}
 // out of 'b' and returns the map and remaining bytes.
 // If 'old' is non-nil, the values will be read into that map.
-func ReadMapStrIntfBytes(b []byte, old map[string]interface{}) (v map[string]interface{}, o []byte, err error) {
+func ReadMapStrIntfBytes(b []byte, old map[string]any) (v map[string]any, o []byte, err error) {
 	return readMapStrIntfBytesDepth(b, old, 0)
 }
 
-func readMapStrIntfBytesDepth(b []byte, old map[string]interface{}, depth int) (v map[string]interface{}, o []byte, err error) {
+func readMapStrIntfBytesDepth(b []byte, old map[string]any, depth int) (v map[string]any, o []byte, err error) {
 	if depth >= recursionLimit {
 		err = ErrRecursion
 		return
@@ -1166,7 +1166,7 @@ func readMapStrIntfBytesDepth(b []byte, old map[string]interface{}, depth int) (
 		}
 		v = old
 	} else {
-		v = make(map[string]interface{}, int(sz))
+		v = make(map[string]any, int(sz))
 	}
 
 	for z := uint32(0); z < sz; z++ {
@@ -1179,7 +1179,7 @@ func readMapStrIntfBytesDepth(b []byte, old map[string]interface{}, depth int) (
 		if err != nil {
 			return
 		}
-		var val interface{}
+		var val any
 		val, o, err = readIntfBytesDepth(o, depth)
 		if err != nil {
 			return
@@ -1192,11 +1192,11 @@ func readMapStrIntfBytesDepth(b []byte, old map[string]interface{}, depth int) (
 // ReadIntfBytes attempts to read
 // the next object out of 'b' as a raw interface{} and
 // return the remaining bytes.
-func ReadIntfBytes(b []byte) (i interface{}, o []byte, err error) {
+func ReadIntfBytes(b []byte) (i any, o []byte, err error) {
 	return readIntfBytesDepth(b, 0)
 }
 
-func readIntfBytesDepth(b []byte, depth int) (i interface{}, o []byte, err error) {
+func readIntfBytesDepth(b []byte, depth int) (i any, o []byte, err error) {
 	if depth >= recursionLimit {
 		err = ErrRecursion
 		return
@@ -1224,7 +1224,7 @@ func readIntfBytesDepth(b []byte, depth int) (i interface{}, o []byte, err error
 			err = ErrShortBytes
 			return
 		}
-		j := make([]interface{}, int(sz))
+		j := make([]any, int(sz))
 		i = j
 		for d := range j {
 			j[d], o, err = readIntfBytesDepth(o, depth+1)
@@ -1283,7 +1283,7 @@ func readIntfBytesDepth(b []byte, depth int) (i interface{}, o []byte, err error
 		}
 		// last resort is a raw extension
 		e := RawExtension{}
-		e.Type = int8(t)
+		e.Type = t
 		o, err = ReadExtensionBytes(b, &e)
 		i = &e
 		return
